local VERSION = tonumber(("$Revision: 45170 $"):match("%d+"))

CowTip = AceLibrary("AceAddon-2.0"):new("AceDB-2.0", "AceEvent-2.0", "AceConsole-2.0", "FuBarPlugin-2.0", "AceHook-2.1", "AceModuleCore-2.0")
local CowTip, self = CowTip, CowTip
CowTip.version = "r" .. VERSION
CowTip.revision = VERSION
CowTip.date = ("$Date: 2007-07-29 04:50:47 -0400 (Sun, 29 Jul 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")

local waterfall = AceLibrary:HasInstance("Waterfall-1.0") and AceLibrary("Waterfall-1.0")

local _G = _G
local GameTooltip = _G.GameTooltip
local geterrorhandler = _G.geterrorhandler
local pcall = _G.pcall
local ipairs = _G.ipairs
local InCombatLockdown = _G.InCombatLockdown
local UIParent = _G.UIParent
local rawget = _G.rawget
local pairs = _G.pairs
local setmetatable = _G.setmetatable
local type = _G.type

local localeTables = {}
function CowTip:L(name)
	if not localeTables[name] then
		localeTables[name] = setmetatable({}, {__index = function(self, key)
			self[key] = key
			return key
		end})
	end
	return localeTables[name]
end

local L = CowTip:L("CowTip")

if GetLocale() == "zhTW" then
	L["Waterfall-1.0 is required to access the GUI."] = "圖形用戶介面設定需要 Waterfall-1.0。";
	L["Never"] = "永不";
	L["Out of combat"] = "戰鬥外";
	L["Always"] = "總是";
	L["Show tooltips"] = "顯示提示訊息";
	L["Show tooltips in specific situations only."] = "只在特定的情況顯示提示訊息。";
	L["World units"] = "世界單位";
	L["Show the tooltip for world units if..."] = "顯示世界單位 (其他玩家，NPC...) 的提示訊息";
	L["World objects"] = "世界物件";
	L["Show the tooltip for world objects if..."] = "顯示世界物件 (郵箱，屍體...) 的提示訊息";
	L["Unit frames"] = "單位框架";
	L["Show the tooltip for unit frames if..."] = "顯示單位框架的提示訊息";
	L["Non-unit frames"] = "非單位框架";
	L["Show the tooltip for non-unit frames if..."] = "顯示非單位框架 (法術，物品...) 的提示訊息";
	L["Enable"] = "啟用";
	L["Enable this module"] = "啟用此模組";
	L["CowTip Configuration"] = "CowTip 設定";
	L["Solo"] = "單練";
	L["Party"] = "隊伍";
	L["Raid"] = "團隊";
	L["Off"] = "關閉";
	L["Configuration mode"] = "設定模式";
	L["|cffffff00Click|r to lock/unlock unit frames."] = "|cffffff00點擊: |r切換單位框架鎖定。";
	L["|cffffff00Shift-Click|r to change configuration mode."] = "|cffffff00Shift-點擊: |r改變設定模式。";
	L["|cffffff00Left-Click|r to change settings with a nice GUI configuration."] = "|cffffff00左擊: |r使用圖形用戶介面設定。";
	L["|cffffff00Right-Click|r to change settings with a drop-down menu."] = "|cffffff00右擊: |r使用下拉選單設定。";
end

CowTip:RegisterDB("CowTipDB")
CowTip:RegisterDefaults('profile', {
	unitShow = "always",
	unitFrameShow = "always",
	otherFrameShow = "always",
	objectShow = "always",
})

-- FuBar configuration
CowTip.hasIcon = true
CowTip.hideWithoutStandby = true
CowTip.defaultPosition = "MINIMAP"
CowTip.defaultMinimapPosition = 230
CowTip.hasNoText = true
CowTip.hasNoColor = true
CowTip.cannotDetachTooltip = true
CowTip.independentProfile = true
CowTip.blizzardTooltip = true

local leftGameTooltipStrings = {}
local rightGameTooltipStrings = {}
CowTip.leftGameTooltipStrings = leftGameTooltipStrings
CowTip.rightGameTooltipStrings = rightGameTooltipStrings

function CowTip:OnInitialize()
	self:RegisterChatCommand('/CowTip', '/Cow', {
		type = 'execute',
		func = function()
			self:OpenMenu()
		end
	})
	
	GameTooltip:SetOwner(UIParent, "ANCHOR_NONE")
	for i = 1, 50 do
		GameTooltip:AddDoubleLine(' ', ' ')
		leftGameTooltipStrings[i] = _G["GameTooltipTextLeft" .. i]
		rightGameTooltipStrings[i] = _G["GameTooltipTextRight" .. i]
	end
	GameTooltip:Show()
	GameTooltip:Hide()
	
	if self.db.profile.texts and self:HasModule("Text") then
		self:GetModule("Text").db.lefts = self.db.profile.texts.lefts
		self:GetModule("Text").db.rights = self.db.profile.texts.rights
		self.db.profile.texts = nil
	end
	
	for k,v in pairs(self.db.profile) do
		for name, module in self:IterateModules() do
			if module.db then
				local db = module.db.profile
				if rawget(db, k) then
					db[k] = v
					self.db.profile[k] = nil
					break
				end
			end
		end
	end
end

function CowTip:OnEnable()
	self:HookScript(GameTooltip, "OnShow", "GameTooltip_OnShow")
	self:HookScript(GameTooltip, "OnHide", "GameTooltip_OnHide")
	self:HookScript(GameTooltip, "OnTooltipSetUnit", "GameTooltip_OnTooltipSetUnit")
	self:HookScript(GameTooltip, "OnTooltipSetItem", "GameTooltip_OnTooltipSetItem")
	self:HookScript(GameTooltip, "OnTooltipSetSpell", "GameTooltip_OnTooltipSetSpell")
	
	self:Hook(GameTooltip, "FadeOut", "GameTooltip_FadeOut", true)
end

function CowTip:OnDisable()
	self:GameTooltip_OnHide(GameTooltip)
end

function CowTip:OnClick()
	if waterfall then
		waterfall:Open("CowTip")
	else
		self:Print(L["Waterfall-1.0 is required to access the GUI."])
	end
end

local function OnTooltipMethod(self, method)
	for i = 1, 3 do
		local methodName
		if i == 1 then
		 	methodName = "PreTooltip" .. method
		elseif i == 2 then
			methodName = "OnTooltip" .. method
		else
			methodName = "PostTooltip" .. method
		end
		self:CallMethodOnAllModules(methodName)
	end
end

local doneOnTooltipMethod = false

local forgetNextOnTooltipMethod = false
function CowTip:GameTooltip_OnTooltipSetUnit(this, ...)
	doneOnTooltipMethod = true
	self.hooks[this].OnTooltipSetUnit(this, ...)
	
	if forgetNextOnTooltipMethod then
		forgetNextOnTooltipMethod = false
	else
		OnTooltipMethod(self, "SetUnit")
	end
end

function CowTip:GameTooltip_OnTooltipSetItem(this, ...)
	forgetNextOnTooltipMethod = true
	self.hooks[this].OnTooltipSetItem(this, ...)
	
	if forgetNextOnTooltipMethod then
		forgetNextOnTooltipMethod = false
	else
		OnTooltipMethod(self, "SetItem")
	end
end

function CowTip:GameTooltip_OnTooltipSetSpell(this, ...)
	doneOnTooltipMethod = true
	self.hooks[this].OnTooltipSetSpell(this, ...)
	
	if forgetNextOnTooltipMethod then
		forgetNextOnTooltipMethod = false
	else
		OnTooltipMethod(self, "SetSpell")
	end
end

function CowTip:GameTooltip_OnShow(this, ...)
	if not doneOnTooltipMethod then
		if this:GetUnit() then
			OnTooltipMethod(self, "SetUnit")
			forgetNextOnTooltipMethod = true
		elseif this:GetItem() then
			OnTooltipMethod(self, "SetItem")
			forgetNextOnTooltipMethod = true
		elseif this:GetSpell() then
			OnTooltipMethod(self, "SetSpell")
			forgetNextOnTooltipMethod = true
		end
	end
	local show
	if this:IsOwned(UIParent) then
		if this:GetUnit() then
			-- world unit
			show = self.db.profile.unitShow
		else
			-- world object
			show = self.db.profile.objectShow
		end
	else
		if this:GetUnit() then
			-- unit frame
			show = self.db.profile.unitFrameShow
		else
			-- non-unit frame
			show = self.db.profile.otherFrameShow
		end
	end
	
	if show == "noncombat" then
		if InCombatLockdown() then
			this:Hide()
			return
		end
	elseif show == "never" then
		this:Hide()
		return
	end
	self.hooks[this].OnShow(this, ...)
	
	self:CallMethodOnAllModules("OnTooltipShow")
end

function CowTip:GameTooltip_FadeOut(this, ...)
	self.hooks[this].FadeOut(this, ...)
end

function CowTip:GameTooltip_OnHide(this, ...)
	doneOnTooltipMethod = false
	forgetNextOnTooltipMethod = false
	
	self:CallMethodOnAllModules("OnTooltipHide")
	
	self.hooks[this].OnHide(this, ...)
end

local showWhenValidate = {
	never = L["Never"],
	notcombat = L["Out of combat"],
	always = L["Always"],
}

local function getShowTooltip(key)
	return CowTip.db.profile[key]
end

local function setShowTooltip(key, value)
	CowTip.db.profile[key] = value
end

CowTip.options = {
	type = 'group',
	handler = CowTip,
	args = {
		showTooltips = {
			type = 'group',
			name = L["Show tooltips"],
			desc = L["Show tooltips in specific situations only."],
			args = {
				unitShow = {
					name = L["World units"],
					desc = L["Show the tooltip for world units if..."],
					type = 'text',
					validate = showWhenValidate,
					get = getShowTooltip,
					set = setShowTooltip,
					passValue = 'unitShow',
				},
				objectShow = {
					name = L["World objects"],
					desc = L["Show the tooltip for world objects if..."],
					type = 'text',
					validate = showWhenValidate,
					get = getShowTooltip,
					set = setShowTooltip,
					passValue = 'objectShow',
				},
				unitFrameShow = {
					name = L["Unit frames"],
					desc = L["Show the tooltip for unit frames if..."],
					type = 'text',
					validate = showWhenValidate,
					get = getShowTooltip,
					set = setShowTooltip,
					passValue = 'unitFrameShow',
				},
				otherFrameShow = {
					name = L["Non-unit frames"],
					desc = L["Show the tooltip for non-unit frames if..."],
					type = 'text',
					validate = showWhenValidate,
					get = getShowTooltip,
					set = setShowTooltip,
					passValue = 'otherFrameShow',
				},
			}
		}
	}
}

function CowTip.modulePrototype:RegisterCowTipOption(data)
	local args = CowTip.options.args
	local name = self.name
	if args[name] then
		local i = 1
		while args[name .. i] do
			i = i + 1
		end
		name = name .. i
	end
	args[name] = data
	if not data.handler then
		data.handler = self
	end
	if data.args then
		data.args.active = {
			type = 'toggle',
			name = L["Enable"],
			desc = L["Enable this module"],
			get = "IsModuleActive",
			set = "ToggleModuleActive",
			handler = CowTip,
			passValue = self,
			order = 1,
		}
	end
end

if waterfall then
	waterfall:Register("CowTip", 
						"aceOptions", CowTip.options,
						"title", L["CowTip Configuration"],
						"treeLevels", 5);
end

CowTip.OnMenuRequest = CowTip.options

function CowTip:OnTooltipUpdate()
	GameTooltip:AddLine("CowTip")
	GameTooltip:AddLine(" ")
	--[[
	GameTooltip:AddDoubleLine(LOCKED .. ":", (self.db.profile.settings.locked and YES or NO), 1, 1, 0, 1, 1, 1)
	local text
	if configMode == "solo" then
		text = L["Solo"]
	elseif configMode == "party" then
		text = L["Party"]
	elseif configMode == "raid" then
		text = L["Raid"]
	else
		text = L["Off"]
	end
	GameTooltip:AddDoubleLine(L["Configuration mode"] .. ":", text, 1, 1, 0, 1, 1, 1)
	GameTooltip:AddLine(" ")
	GameTooltip:AddLine(L["|cffffff00Click|r to lock/unlock unit frames."], 0.2, 1, 0.2)
	GameTooltip:AddLine(L["|cffffff00Shift-Click|r to change configuration mode."], 0.2, 1, 0.2)
	]]
	if waterfall then
		GameTooltip:AddLine(L["|cffffff00Left-Click|r to change settings with a nice GUI configuration."], 0.2, 1, 0.2)
	end
	GameTooltip:AddLine(L["|cffffff00Right-Click|r to change settings with a drop-down menu."], 0.2, 1, 0.2)
end
