if select(6, GetAddOnInfo("CowTip_" .. (debugstack():match("[C%.][o%.][w%.]Tip\\Modules\\(.-)\\") or debugstack():match("[C%.][o%.][w%.]Tip\\(.-)\\") or ""))) ~= "MISSING" then return end

local VERSION = tonumber(("$Revision: 45170 $"):match("%d+"))

local CowTip = CowTip
local CowTip_Fade = CowTip:NewModule("Fade", "AceEvent-2.0", "AceHook-2.1")
local self = CowTip_Fade
if CowTip.revision < VERSION then
	CowTip.version = "r" .. VERSION
	CowTip.revision = VERSION
	CowTip.date = ("$Date: 2007-07-29 04:50:47 -0400 (Sun, 29 Jul 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end
CowTip_Fade.desc = "Allow for positioning of the tooltip"

local L = CowTip:L("CowTip_Fade")

if GetLocale() == "zhTW" then
	L["Immediate hide"] = "立即隱藏";
	L["Fade out"] = "淡出";
	L["Fade"] = "消失";
	L["Change how the tooltip fades."] = "更改提示訊息消失方法。";
	L["World units"] = "世界單位";
	L["What kind of fade to use for world units (other players, non-player characters in the world, etc.)"] = "世界單位 (其他玩家，NPC...) 的消失方法。";
	L["World objects"] = "世界物件";
	L["What kind of fade to use for world objects (mailboxes, corpses, etc.)"] = "世界物件 (郵箱，屍體...) 的消失方法。";
	L["Unit frames"] = "單位框架";
	L["What kind of fade to use for unit frames."] = "單位框架的消失方法。";
	L["Non-unit frames"] = "非單位框架";
	L["What kind of fade to use for non-unit frames (spells, items, etc.)"] = "非單位框架 (法術，物品...) 的消失方法。";
end

local _G = _G
local GetScreenWidth = _G.GetScreenWidth
local GetScreenHeight = _G.GetScreenHeight
local UIParent = _G.UIParent
local CreateFrame = _G.CreateFrame
local GetCursorPosition = _G.GetCursorPosition
local UnitExists = _G.UnitExists
local GameTooltip = _G.GameTooltip

function CowTip_Fade:OnInitialize()
	self.db = CowTip:AcquireDBNamespace("Fade")
	CowTip:RegisterDefaults("Fade", "profile", {
		units = "fade",
		objects = "fade",
		unitFrames = "hide",
		otherFrames = "hide",
	})
end

function CowTip_Fade:OnEnable()
	self:Hook(GameTooltip, 'Hide', "GameTooltip_Hide", true)
	self:Hook(GameTooltip, 'FadeOut', "GameTooltip_FadeOut", true)
	
	self:RegisterEvent("CURSOR_UPDATE")
end

function CowTip_Fade:GameTooltip_Hide(this, ...)
	local kind
	if GameTooltip:GetUnit() then
		if GameTooltip:IsOwned(UIParent) then -- world unit
			kind = CowTip_Fade.db.profile.units
		else
			kind = CowTip_Fade.db.profile.unitFrames
		end
	else
		if GameTooltip:IsOwned(UIParent) then -- world object
			kind = CowTip_Fade.db.profile.objects
		else
			kind = CowTip_Fade.db.profile.otherFrames
		end
	end
	if kind == "fade" then
		return GameTooltip:FadeOut()
	else
		return self.hooks[this].Hide(this, ...)
	end
end

function CowTip_Fade:GameTooltip_FadeOut(this, ...)
	local kind
	if GameTooltip:GetUnit() then
		if GameTooltip:IsOwned(UIParent) then -- world unit
			kind = CowTip_Fade.db.profile.units
		else
			kind = CowTip_Fade.db.profile.unitFrames
		end
	else
		if GameTooltip:IsOwned(UIParent) then -- world object
			kind = CowTip_Fade.db.profile.objects
		else
			kind = CowTip_Fade.db.profile.otherFrames
		end
	end
	if kind == "fade" then
		self.hooks[this].FadeOut(this, ...)
	else
		GameTooltip:Hide()
	end
end

local function CheckUnitExistence()
	if not GameTooltip:GetUnit() or not UnitExists("mouseover") then
		self:CancelScheduledEvent("CowTip_Fade_CheckExistence")
		local kind
		if GameTooltip:IsOwned(UIParent) then -- world unit
			kind = CowTip_Fade.db.profile.units
		else -- unit frames
			kind = CowTip_Fade.db.profile.unitFrames
		end
		if kind == "fade" then
			GameTooltip:FadeOut()
		else
			GameTooltip:Hide()
		end
	end
end

local function CheckTooltipAlpha()
	if GameTooltip:GetAlpha() < 1 then
		self:CancelScheduledEvent("CowTip_Fade_CheckExistence")
		
		local kind
		if GameTooltip:IsOwned(UIParent) then -- world object
			kind = CowTip_Fade.db.profile.objects
		else -- unit frames
			kind = CowTip_Fade.db.profile.otherFrames
		end
		if kind == "fade" then
			GameTooltip:FadeOut()
		else
			GameTooltip:Hide()
		end
	end
end

local cursorChangedWithTooltip = false

function CowTip_Fade:OnTooltipShow()
	if GameTooltip:GetUnit() then
		self:ScheduleRepeatingEvent("CowTip_Fade_CheckExistence", CheckUnitExistence, 0)
	else
		if GameTooltip:IsOwned(UIParent) and self:IsEventScheduled("CowTip_Fade_CursorUpdate") then
			cursorChangedWithTooltip = true
		end
		self:ScheduleRepeatingEvent("CowTip_Fade_CheckExistence", CheckTooltipAlpha, 0)
	end
end

function CowTip_Fade:OnTooltipHide()
	cursorChangedWithTooltip = nil
	self:CancelScheduledEvent("CowTip_Fade_CheckExistence")
end

local function donothing() end

function CowTip_Fade:CURSOR_UPDATE(...)
	if cursorChangedWithTooltip then
		if CowTip_Fade.db.profile.objects == "fade" then
			GameTooltip:FadeOut()
		else
			GameTooltip:Hide()
		end
	else
		self:ScheduleEvent("CowTip_Fade_CursorUpdate", donothing, 0)
	end
end

local validate = {
	hide = L["Immediate hide"],
	fade = L["Fade out"],
}

local function getFadeKind(style)
	return CowTip_Fade.db.profile[style]
end

local function setFadeKind(style, value)
	CowTip_Fade.db.profile[style] = value
end

CowTip_Fade:RegisterCowTipOption({
	name = L["Fade"],
	desc = L["Change how the tooltip fades."],
	type = 'group',
	args = {
		units = {
			name = L["World units"],
			desc = L["What kind of fade to use for world units (other players, non-player characters in the world, etc.)"],
			type = 'text',
			validate = validate,
			get = getFadeKind,
			set = setFadeKind,
			passValue = "units",
		},
		objects = {
			name = L["World objects"],
			desc = L["What kind of fade to use for world objects (mailboxes, corpses, etc.)"],
			type = 'text',
			validate = validate,
			get = getFadeKind,
			set = setFadeKind,
			passValue = "objects",
		},
		unitFrames = {
			name = L["Unit frames"],
			desc = L["What kind of fade to use for unit frames."],
			type = 'text',
			validate = validate,
			get = getFadeKind,
			set = setFadeKind,
			passValue = "unitFrames",
		},
		otherFrames = {
			name = L["Non-unit frames"],
			desc = L["What kind of fade to use for non-unit frames (spells, items, etc.)"],
			type = 'text',
			validate = validate,
			get = getFadeKind,
			set = setFadeKind,
			passValue = "otherFrames",
		},
	}
})
