if select(6, GetAddOnInfo("CowTip_" .. (debugstack():match("[C%.][o%.][w%.]Tip\\Modules\\(.-)\\") or debugstack():match("[C%.][o%.][w%.]Tip\\(.-)\\") or ""))) ~= "MISSING" then return end

local VERSION = tonumber(("$Revision: 45170 $"):match("%d+"))

local CowTip = CowTip
local CowTip_Position = CowTip:NewModule("Position", "AceEvent-2.0", "AceHook-2.1")
local self = CowTip_Position
if CowTip.revision < VERSION then
	CowTip.version = "r" .. VERSION
	CowTip.revision = VERSION
	CowTip.date = ("$Date: 2007-07-29 04:50:47 -0400 (Sun, 29 Jul 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end
CowTip_Position.desc = "Allow for positioning of the tooltip"

local L = CowTip:L("CowTip_Position")

if GetLocale() == "zhTW" then
	L["%s, %s"] = "%s，%s";
	L["Cursor"] = "鼠標";
	L["Screen"] = "螢幕";
	L["Top-left"] = "左上";
	L["Top"] = "上";
	L["Top-right"] = "右上";
	L["Left"] = "左";
	L["Middle"] = "中";
	L["Right"] = "右";
	L["Bottom-left"] = "左下";
	L["Bottom"] = "下";
	L["Bottom-right"] = "右下";
	L["Anchor: %s\nClick and drag to the position you want.\nDouble-click when finished."] = "錨點: %s\n點擊和拖動到適當的位置。\n雙擊確認。";
	L["Position"] = "位置";
	L["Change where the tooltip is located."] = "更改提示訊息的位置。";
	L["Unit anchor"] = "單位錨點";
	L["The anchor with which unit mouseover tooltips are located."] = "設定單位提示訊息的錨點。";
	L["Set unit anchor offset"] = "設定單位錨點位移";
	L["Click this to bring up an easy-to-use offset box."] = "點擊打開易用位移輸入框。";
	L["Frame anchor"] = "框架錨點";
	L["The anchor with which tooltips are located when hovering over frames."] = "設定框架提示訊息的錨點。";
	L["Set frame anchor offset"] = "設定框架錨點位移";
end

local _G = _G
local GetScreenWidth = _G.GetScreenWidth
local GetScreenHeight = _G.GetScreenHeight
local UIParent = _G.UIParent
local CreateFrame = _G.CreateFrame
local GetCursorPosition = _G.GetCursorPosition
local GameTooltip = _G.GameTooltip

function CowTip_Position:OnInitialize()
	self.db = CowTip:AcquireDBNamespace("Position")
	CowTip:RegisterDefaults("Position", "profile", {
		unitAnchor = "CURSOR_BOTTOM",
		unitOffsetX = 0,
		unitOffsetY = 0,
		frameAnchor = "BOTTOMRIGHT",
		frameOffsetX = 0,
		frameOffsetY = 0,
	})
end

function CowTip_Position:OnEnable()
	self:Hook("GameTooltip_SetDefaultAnchor", "GameTooltip_SetDefaultAnchor", true)
end

function CowTip_Position:OnTooltipHide()
	self:CancelScheduledEvent("CowTip_Position_Update")
end

local currentOffsetX, currentOffsetY = 0, 0
local currentCursorAnchor = "BOTTOM"

local function ReanchorTooltip()
	GameTooltip:ClearAllPoints()
	local x, y = GetCursorPosition()
	local scale = GameTooltip:GetEffectiveScale()
	x, y = (x + currentOffsetX) / scale, (y + currentOffsetY) / scale
	GameTooltip:SetPoint(currentCursorAnchor, UIParent, "BOTTOMLEFT", x, y)
end

function CowTip_Position:GameTooltip_SetDefaultAnchor(this, owner, ...)
	self.hooks.GameTooltip_SetDefaultAnchor(this, owner, ...)
	
	local db = self.db.profile
	
	local anchor, offsetX, offsetY
	if owner == UIParent then -- world character
		anchor = db.unitAnchor
		offsetX = db.unitOffsetX
		offsetY = db.unitOffsetY
	else -- frame
		anchor = db.frameAnchor
		offsetX = db.frameOffsetX
		offsetY = db.frameOffsetY
	end
	if anchor:find("^CURSOR") then
		currentOffsetX = offsetX
		currentOffsetY = offsetY
		currentCursorAnchor = anchor:sub(8)
		self:ScheduleRepeatingEvent("CowTip_Position_Update", ReanchorTooltip, 0)
	else
		this:SetOwner(owner, "ANCHOR_NONE")
		this:ClearAllPoints()
		this:SetPoint(anchor, UIParent, anchor, offsetX, offsetY)
	end
end

local offsetBox

function CowTip_Position:SetUnitAnchor(value)
	if offsetBox then
		offsetBox:GetScript("OnDoubleClick")(offsetBox, "LeftButton")
	end
	self.db.profile.unitAnchor = value
	self.db.profile.unitOffsetX = 0
	self.db.profile.unitOffsetY = 0
end

function CowTip_Position:SetFrameAnchor(value)
	if offsetBox then
		offsetBox:GetScript("OnDoubleClick")(offsetBox, "LeftButton")
	end
	self.db.profile.frameAnchor = value
	self.db.profile.frameOffsetX = 0
	self.db.profile.frameOffsetY = 0
end

local anchorValidate = {
	CURSOR_BOTTOM = L["%s, %s"]:format(L["Cursor"], L["Top"]),
	CURSOR_BOTTOMRIGHT = L["%s, %s"]:format(L["Cursor"], L["Top-left"]),
	CURSOR_BOTTOMLEFT = L["%s, %s"]:format(L["Cursor"], L["Top-right"]),
	CURSOR_LEFT = L["%s, %s"]:format(L["Cursor"], L["Right"]),
	CURSOR_RIGHT = L["%s, %s"]:format(L["Cursor"], L["Left"]),
	CURSOR_TOP = L["%s, %s"]:format(L["Cursor"], L["Bottom"]),
	CURSOR_TOPRIGHT = L["%s, %s"]:format(L["Cursor"], L["Bottom-left"]),
	CURSOR_TOPLEFT = L["%s, %s"]:format(L["Cursor"], L["Bottom-right"]),
	TOPLEFT = L["%s, %s"]:format(L["Screen"], L["Top-left"]),
	TOP = L["%s, %s"]:format(L["Screen"], L["Top"]),
	TOPRIGHT = L["%s, %s"]:format(L["Screen"], L["Top-right"]),
	LEFT = L["%s, %s"]:format(L["Screen"], L["Left"]),
	RIGHT = L["%s, %s"]:format(L["Screen"], L["Right"]),
	BOTTOMLEFT = L["%s, %s"]:format(L["Screen"], L["Bottom-left"]),
	BOTTOM = L["%s, %s"]:format(L["Screen"], L["Bottom"]),
	BOTTOMRIGHT = L["%s, %s"]:format(L["Screen"], L["Bottom-right"]),
	CENTER = L["%s, %s"]:format(L["Screen"], L["Middle"]),
}

local fakeMouse
function CowTip_Position:CreateOffsetBox(kind)
	if not offsetBox then
		offsetBox = CreateFrame("Button", "CowTipOffsetBox", UIParent)
		offsetBox:SetWidth(300)
		offsetBox:SetHeight(100)
		offsetBox:SetFrameStrata("TOOLTIP")
		
		local bg = offsetBox:CreateTexture("CowTipOffsetBoxBackground", "BACKGROUND")
		bg:SetTexture(0.7, 0.4, 0) -- orange
		bg:SetAllPoints(offsetBox)
		
		local text = offsetBox:CreateFontString("CowTipOffsetBoxText", "ARTWORK", "GameFontHighlight")
		offsetBox.text = text
		text:SetPoint("CENTER")
		
		offsetBox:SetScript("OnDoubleClick", function(this)
			-- cleanup
			this:StopMovingOrSizing()
			this:Hide()
			fakeMouse:Hide()
		end)
		
		offsetBox:SetScript("OnDragStart", function(this)
			this:StartMoving()
		end)
		
		offsetBox:SetScript("OnDragStop", function(this)
			local kind = this.kind
			local db = self.db.profile
			local anchor
			if kind == "unit" then
				anchor = db.unitAnchor
			else
				anchor = db.frameAnchor
			end
			local offsetX, offsetY
			if anchor == "BOTTOMLEFT" or anchor == "BOTTOM" or anchor == "BOTTOMRIGHT" then
				offsetY = this:GetBottom()
			elseif anchor == "LEFT" or anchor == "CENTER" or anchor == "RIGHT" then
				local _,y = this:GetCenter()
				offsetY = y - GetScreenHeight()/2
			elseif anchor == "TOPLEFT" or anchor == "TOP" or anchor == "TOPRIGHT" then
				offsetY = this:GetTop() - GetScreenHeight()
			elseif anchor == "CURSOR_BOTTOM" or anchor == "CURSOR_BOTTOMLEFT" or anchor == "CURSOR_BOTTOMRIGHT" then
				offsetY = this:GetBottom() - GetScreenHeight()/2
			elseif anchor == "CURSOR_TOP" or anchor == "CURSOR_TOPLEFT" or anchor == "CURSOR_TOPRIGHT" then
				offsetY = this:GetTop() - GetScreenHeight()/2
			elseif anchor == "CURSOR_LEFT" or anchor == "CURSOR_RIGHT" then
				local _,y = this:GetCenter()
				offsetY = y - GetScreenHeight()/2
			end
			if anchor == "TOPLEFT" or anchor == "LEFT" or anchor == "BOTTOMLEFT" then
				offsetX = this:GetLeft()
			elseif anchor == "TOP" or anchor == "CENTER" or anchor == "BOTTOM" then
				offsetX = this:GetCenter() - GetScreenWidth()/2
			elseif anchor == "TOPRIGHT" or anchor == "RIGHT" or anchor == "BOTTOMRIGHT" then
				offsetX = this:GetRight() - GetScreenWidth()
			elseif anchor == "CURSOR_RIGHT" or anchor == "CURSOR_TOPRIGHT" or anchor == "CURSOR_BOTTOMRIGHT" then
				offsetX = this:GetRight() - GetScreenWidth()/2
			elseif anchor == "CURSOR_LEFT" or anchor == "CURSOR_TOPLEFT" or anchor == "CURSOR_BOTTOMLEFT" then
				offsetX = this:GetLeft() - GetScreenWidth()/2
			elseif anchor == "CURSOR_TOP" or anchor == "CURSOR_BOTTOM" then
				offsetX = this:GetCenter() - GetScreenWidth()/2
			end
			if kind == "unit" then
				db.unitOffsetX = offsetX
				db.unitOffsetY = offsetY
			else
				db.frameOffsetX = offsetX
				db.frameOffsetY = offsetY
			end
			-- save
			this:StopMovingOrSizing()
		end)
		
		offsetBox:SetMovable(true)
		offsetBox:RegisterForDrag("LeftButton")
		offsetBox:Hide()
		
		fakeMouse = CreateFrame("Frame", "CowTipOffsetBoxFakeMouse", UIParent)
		fakeMouse:SetWidth(1)
		fakeMouse:SetHeight(1)
		fakeMouse:SetPoint("CENTER")
		local tex = fakeMouse:CreateTexture(nil, "ARTWORK")
		tex:SetTexture("Interface\\CURSOR\\Cast")
		tex:SetPoint("TOPLEFT", fakeMouse, "TOPLEFT")
		fakeMouse:Hide()
	end
	
	offsetBox:Show()
	offsetBox:SetScale(GameTooltip:GetScale())
	
	local anchor, offsetX, offsetY
	offsetBox.kind = kind
	local db = self.db.profile
	if kind == "unit" then
		anchor = db.unitAnchor
		offsetX = db.unitOffsetX
		offsetY = db.unitOffsetY
	else
		anchor = db.frameAnchor
		offsetX = db.frameOffsetX
		offsetY = db.frameOffsetY
	end
	
	offsetBox:ClearAllPoints()
	offsetBox.text:SetText(L["Anchor: %s\nClick and drag to the position you want.\nDouble-click when finished."]:format(anchorValidate[anchor]))
	if anchor:find("^CURSOR") then
		offsetBox:SetPoint(anchor:sub(8), UIParent, "CENTER", offsetX, offsetY)
		fakeMouse:Show()
	else
		offsetBox:SetPoint(anchor, UIParent, anchor, offsetX, offsetY)
	end
	offsetBox:SetClampedToScreen(true)
end

CowTip_Position:RegisterCowTipOption({
	name = L["Position"],
	desc = L["Change where the tooltip is located."],
	type = 'group',
	args = {
		unitAnchor = {
			type = 'text',
			name = L["Unit anchor"],
			desc = L["The anchor with which unit mouseover tooltips are located."],
			get = function()
				return CowTip_Position.db.profile.unitAnchor
			end,
			set = "SetUnitAnchor",
			validate = anchorValidate,
			order = 51,
		},
		unitOffset = {
			type = 'execute',
			name = L["Set unit anchor offset"],
			desc = L["Click this to bring up an easy-to-use offset box."],
			func = "CreateOffsetBox",
			passValue = "unit",
			order = 52,
		},
		frameAnchor = {
			type = 'text',
			name = L["Frame anchor"],
			desc = L["The anchor with which tooltips are located when hovering over frames."],
			get = function()
				return CowTip_Position.db.profile.frameAnchor
			end,
			set = "SetFrameAnchor",
			validate = anchorValidate,
			order = 53,
		},
		frameOffset = {
			type = 'execute',
			name = L["Set frame anchor offset"],
			desc = L["Click this to bring up an easy-to-use offset box."],
			func = "CreateOffsetBox",
			passValue = "frame",
			order = 54,
		},
	}
})