if select(6, GetAddOnInfo("CowTip_" .. (debugstack():match("[C%.][o%.][w%.]Tip\\Modules\\(.-)\\") or debugstack():match("[C%.][o%.][w%.]Tip\\(.-)\\") or ""))) ~= "MISSING" then return end

local VERSION = tonumber(("$Revision: 45170 $"):match("%d+"))

local CowTip = CowTip
local CowTip_PowerBar = CowTip:NewModule("PowerBar", "AceEvent-2.0")
local self = CowTip_PowerBar
if CowTip.revision < VERSION then
	CowTip.version = "r" .. VERSION
	CowTip.revision = VERSION
	CowTip.date = ("$Date: 2007-07-29 04:50:47 -0400 (Sun, 29 Jul 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end
CowTip_PowerBar.desc = "Allow for a power bar on the tooltip"

local L = CowTip:L("CowTip_PowerBar")

if GetLocale() == "zhTW" then
	L["Power bar"] = "能力條";
	L["Change settings for the power bar."] = "更改能力條設定。";
	L["Size"] = "大小";
	L["The size of the power bar."] = "能力條的大小。";
	L["Position"] = "位置";
	L["The position of the power bar relative to the tooltip."] = "能力條相對於提示訊息的位置。";
	L["Left"] = "左";
	L["Right"] = "右";
	L["Top"] = "上";
	L["Bottom"] = "下";
	L["Texture"] = "紋理";
	L["The texture which the power bar uses."] = "能力條的紋理。";
end

local SharedMedia = AceLibrary("SharedMedia-1.0")

local _G = _G
local UnitIsUnit = _G.UnitIsUnit
local CreateFrame = _G.CreateFrame
local GameTooltip = _G.GameTooltip
local UnitManaMax = _G.UnitManaMax
local UnitMana = _G.UnitMana
local UnitPowerType = _G.UnitPowerType

function CowTip_PowerBar:OnInitialize()
	self.db = CowTip:AcquireDBNamespace("PowerBar")
	CowTip:RegisterDefaults("PowerBar", "profile", {
		size = 5,
		position = "BOTTOM",
		texture = "Blizzard",
	})
end

local powerBar
function CowTip_PowerBar:OnEnable(first)
	self:RegisterEvent("UNIT_MANA")
	self:RegisterEvent("UNIT_RAGE", "UNIT_MANA")
	self:RegisterEvent("UNIT_FOCUS", "UNIT_MANA")
	self:RegisterEvent("UNIT_ENERGY", "UNIT_MANA")
	self:RegisterEvent("UNIT_MAXMANA", "UNIT_MANA")
	self:RegisterEvent("UNIT_MAXRAGE", "UNIT_MANA")
	self:RegisterEvent("UNIT_MAXFOCUS", "UNIT_MANA")
	self:RegisterEvent("UNIT_MAXENERGY", "UNIT_MANA")
	self:RegisterEvent("UNIT_DISPLAYPOWER", "UNIT_MANA")
	
	self:Reposition()
	if CowTip:HasModule("HealthBar") and CowTip:IsModuleActive("HealthBar") then
		CowTip:GetModule("HealthBar"):Reposition()
	end
end

function CowTip_PowerBar:OnDisable()
	if powerBar then
		powerBar:Hide()
		powerBar.side = nil
	end
	
	if CowTip:HasModule("HealthBar") and CowTip:IsModuleActive("HealthBar") then
		CowTip:GetModule("HealthBar"):Reposition()
	end
end

function CowTip_PowerBar:UNIT_MANA(unit)
	if not UnitIsUnit(unit, "mouseover") then
		return
	end
	
	self:Update()
end

function CowTip_PowerBar:OnTooltipShow()
	if not powerBar then
		powerBar = CreateFrame("StatusBar", "CowTip_PowerBar_Bar", GameTooltip)
		powerBar:SetStatusBarTexture(SharedMedia:Fetch('statusbar', self.db.profile.texture))
		powerBar:SetMinMaxValues(0, 1)
		self:Reposition()
		if CowTip:HasModule("HealthBar") and CowTip:IsModuleActive("HealthBar") then
			CowTip:GetModule("HealthBar"):Reposition()
		end
	end
	
	if not GameTooltip:GetUnit() then
		powerBar:Hide()
		return
	end
	
	powerBar:Show()
	
	self:Update()
end

function CowTip_PowerBar:OnTooltipHide()
	if not powerBar then
		return
	end
	powerBar:Hide()
end

function CowTip_PowerBar:Update()
	if not powerBar then
		return
	end
	local max = UnitManaMax("mouseover")
	local value
	if max == 0 then
		value = 0
	else
		value = UnitMana("mouseover") / max
	end
	powerBar:SetValue(value)
	local powerType = UnitPowerType("mouseover")
	if powerType == 0 then
		-- mana
		powerBar:SetStatusBarColor(48/255, 113/255, 191/255)
	elseif powerType == 1 then
		-- rage
		powerBar:SetStatusBarColor(226/255, 45/255, 75/255)
	elseif powerType == 2 then
		-- focus
		powerBar:SetStatusBarColor(1, 210/255, 0)
	else
		-- energy
		powerBar:SetStatusBarColor(1, 220/255, 25/255)
	end
end

function CowTip_PowerBar:SetSize(size)
	self.db.profile.size = size
	if not powerBar then
		return
	end
	powerBar:SetHeight(size)
end

function CowTip_PowerBar:Reposition()
	if not CowTip:IsModuleActive(self) or not powerBar then
		return
	end
	local position = self.db.profile.position
	
	powerBar:SetWidth(0)
	powerBar:SetHeight(0)
	powerBar:ClearAllPoints()
	powerBar.side = position
	if position == "TOP" then
		powerBar:SetPoint("BOTTOMLEFT", GameTooltip, "TOPLEFT", 4, 0)
		powerBar:SetPoint("BOTTOMRIGHT", GameTooltip, "TOPRIGHT", -4, 0)
		powerBar:SetHeight(self.db.profile.size)
		powerBar:SetOrientation("HORIZONTAL")
	elseif position == "BOTTOM" then
		local healthBar = _G.CowTip_HealthBar_Bar
		if healthBar and healthBar.side == "BOTTOM" then
			powerBar:SetPoint("TOPLEFT", healthBar, "BOTTOMLEFT")
			powerBar:SetPoint("TOPRIGHT", healthBar, "BOTTOMRIGHT")
		else
			powerBar:SetPoint("TOPLEFT", GameTooltip, "BOTTOMLEFT", 4, 0)
			powerBar:SetPoint("TOPRIGHT", GameTooltip, "BOTTOMRIGHT", -4, 0)
		end
		powerBar:SetHeight(self.db.profile.size)
		powerBar:SetOrientation("HORIZONTAL")
	elseif position == "LEFT" then
		powerBar:SetPoint("TOPRIGHT", GameTooltip, "TOPLEFT", 0, -4)
		powerBar:SetPoint("BOTTOMRIGHT", GameTooltip, "BOTTOMLEFT", 0, 4)
		powerBar:SetWidth(self.db.profile.size)
		powerBar:SetOrientation("VERTICAL")
	elseif position == "RIGHT" then
		local healthBar = _G.CowTip_HealthBar_Bar
		if healthBar and healthBar.side == "RIGHT" then
			powerBar:SetPoint("TOPLEFT", healthBar, "TOPRIGHT")
			powerBar:SetPoint("BOTTOMLEFT", healthBar, "BOTTOMRIGHT")
		else
			powerBar:SetPoint("TOPLEFT", GameTooltip, "TOPRIGHT", 0, -4)
			powerBar:SetPoint("BOTTOMLEFT", GameTooltip, "BOTTOMRIGHT", 0, 4)
		end
		powerBar:SetWidth(self.db.profile.size)
		powerBar:SetOrientation("VERTICAL")
	end
end

function CowTip_PowerBar:SetPosition(position)
	self.db.profile.position = position
	if not powerBar then
		return
	end
	self:Reposition()
	if CowTip:HasModule("HealthBar") and CowTip:IsModuleActive("HealthBar") then
		CowTip:GetModule("HealthBar"):Reposition()
	end
end

function CowTip_PowerBar:SetTexture(value)
	self.db.profile.texture = value
	if not powerBar then
		return
	end
	powerBar:SetStatusBarTexture(SharedMedia:Fetch('statusbar', value))
end

CowTip_PowerBar:RegisterCowTipOption({
	name = L["Power bar"],
	desc = L["Change settings for the power bar."],
	type = 'group',
	args = {
		size = {
			type = 'range',
			name = L["Size"],
			desc = L["The size of the power bar."],
			get = function()
				return CowTip_PowerBar.db.profile.size
			end,
			set = "SetSize",
			min = 1,
			max = 20,
			step = 1,
		},
		position = {
			type = 'text',
			name = L["Position"],
			desc = L["The position of the power bar relative to the tooltip."],
			get = function()
				return CowTip_PowerBar.db.profile.position
			end,
			set = "SetPosition",
			validate = {
				LEFT = L["Left"],
				RIGHT = L["Right"],
				TOP = L["Top"],
				BOTTOM = L["Bottom"],
			}
		},
		texture = {
			type = 'text',
			name = L["Texture"],
			desc = L["The texture which the power bar uses."],
			get = function()
				return CowTip_PowerBar.db.profile.texture
			end,
			set = "SetTexture",
			validate = SharedMedia:List('statusbar'),
		},
	}
})